import numpy as np
import os
import sys

#print("This script only works with python2.")

seqFilename = "sequence.txt"
param = 2.5 #Expected similarity
cliqueOut = "clique.txt"
cliqueSequence = "final_list.txt"


if len(sys.argv) > 1:
    seqFilename = sys.argv[1]

if len(sys.argv) > 2:
    param = float(sys.argv[2])

if len(sys.argv) > 3:
    cliqueOut = sys.argv[3]

if len(sys.argv) > 4:
    cliqueSequence = sys.argv[4]

if len(sys.argv) > 5:
    print("This script takes at most four arguments.")
    print("(1) Input file name (sequence.txt is default)")
    print("(2) Expected similarity (2.5 is default)")
    print("(3) File name to save clique (clicque.txt is default)")
    print("(4) File name to save clique sequneces (clicque_sequence.txt is default)")
    quit()

if len(sys.argv)>1 and (sys.argv[1]=="help" or sys.argv[1]=="--help" or sys.argv[1]=="-h"):
    print("This script takes at most four optional arguments:")
    print("(1) Input file name (sequence.txt is default)")
    print("(2) Expected similarity (2.5 is default)")
    print("(3) File name to save clique (clique.txt is default)")
    print("(4) File name to save clique sequneces (clique_sequence.txt is default)")
    quit()

f = open(seqFilename,"r")
seq = f.readlines()
f.close()


for i in range(len(seq)):
    seq[i]=seq[i].rstrip()



#Bron-Kerbosch algorithm for clique finding
def bronk2(R, P, X, g):
    if len(P) == 0 and len(X) == 0:
        f.write(' %s\n ' % sorted(R))
        return 
    
    #We find vertices with most ones
    PUX=[]
    for v in P[:]:
        PUX.append(v)
    for i in X[:]:
        PUX.append(i)
    PUX.sort()
    g1=[]
    for i in PUX:
        g1.append(g[i])
    M=[]
    l=[]
    for j in g1:
        M.append(sum(j))
    m=max(M)
    for j in range(len(M)):
        if(M[j]==m):
            l.append(g.index(g1[j]))
      
    #We choose pivot vertex
    u=l[0]
    
    #We find P\N(u,g)
    A=[]
    for v in P[:]:
        if (v not in N(u,g)):
            A.append(v)
    
    for v in A[:]:
        R_v = R + [v]
        P_v=[]
        for v1 in P[:] :
            if (v1 in N(v, g)):
                P_v.append(v1)
        X_v=[]
        for v1 in X[:] :
            if (v1 in N(v, g)):
                X_v.append(v1)
        bronk2(R_v, P_v, X_v, g)
        P.remove(v)
        X.append(v)


def N(v, g):
    c = 0
    l = []
    for i in g[v]:
        if(i==1):
            l.append(c)
        c=c+1   
    return l

## amino acids
ak = "ARNDCQEGHILKMFPSTWYV"

## blosum matrix
m = [[5,-2,-1,-2,-1,-1,-1,0,-2,-1,-2,-1,-1,-3,-1,1,0,-3,-2,0], 
     [-2,7,-1,-2,-4,1,0,-3,0,-4,-3,3,-2,-3,-3,-1,-1,-3,-1,-3],
     [-1,-1,7,2,-2,0,0,0,1,-3,-4,0,-2,-4,-2,1,0,-4,-2,-3], 
     [-2,-2,2,8,-4,0,2,-1,-1,-4,-4,-1,-4,-5,-1,0,-1,-5,-3,-4], 
     [-1,-4,-2,-4,13,-3,-3,-3,-3,-2,-2,-3,-2,-2,-4,-1,-1,-5,-3,-1], 
     [-1,1,0,0,-3,7,2,-2,1,-3,-2,2,0,-4,-1,0,-1,-1,-1,-3],
     [-1,0,0,2,-3,2,6,-3,0,-4,-3,1,-2,-3,-1,-1,-1,-3,-2,-3],
     [0,-3,0,-1,-3,-2,-3,8,-2,-4,-4,-2,-3,-4,-2,0,-2,-3,-3,-4],
     [-2,0,1,-1,-3,1,0,-2,10,-4,-3,0,-1,-1,-2,-1,-2,-3,2,-4],
     [-1,-4,-3,-4,-2,-3,-4,-4,-4,5,2,-3,2,0,-3,-3,-1,-3,-1,4],
     [-2,-3,-4,-4,-2,-2,-3,-4,-3,2,5,-3,3,1,-4,-3,-1,-2,-1,1],
     [-1,3,0,-1,-3,2,1,-2,0,-3,-3,6,-2,-4,-1,0,-1,-3,-2,-3],
     [-1,-2,-2,-4,-2,0,-2,-3,-1,2,3,-2,7,0,-3,-2,-1,-1,0,1],
     [-3,-3,-4,-5,-2,-4,-3,-4,-1,0,1,-4,0,8,-4,-3,-2,1,4,-1],
     [-1,-3,-2,-1,-4,-1,-1,-2,-2,-3,-4,-1,-3,-4,10,-1,-1,-4,-3,-3],
     [1,-1,1,0,-1,0,-1,0,-1,-3,-3,0,-2,-3,-1,5,2,-4,-2,-2],
     [0,-1,0,-1,-1,-1,-1,-2,-2,-1,-1,-1,-1,-2,-1,2,5,-3,-2,0],
     [-3,-3,-4,-5,-5,-1,-3,-3,-3,-3,-2,-3,-1,1,-4,-4,-3,15,2,-3],
     [-2,-1,-2,-3,-3,-1,-2,-3,2,-1,-1,-2,0,4,-3,-2,-2,2,8,-1],
     [0,-3,-3,-4,-1,-3,-3,-4,-4,4,1,-3,1,-1,-3,-2,0,-3,-1,5]]



##reading motifs
lm=seq
k=len(lm)
mw=len(lm[0])
sm=[]
for i in range(k):
    tmp=[]
    for j in range(k):
        s=0
        for l in range(mw):
            a=str(lm[i][l])
            b=str(lm[j][l])
            aa=ak.index(a)
            bb=ak.index(b)
            s=s+m[aa][bb]
        tmp.append(s)
    sm.append(tmp[:])


#threshold for 0-1 graf
threshold = param * len(lm[0])

## computing 0-1 matrix
matrix=sm
for i in range(k):
    for j in range(k):
        if matrix[i][j]<threshold:
            matrix[i][j]=0
        else:
            matrix[i][j]=1

for i in range(k):
    matrix[i][i]=0        

try:
    f=open("temp.txt", "w")
    bronk2([],range(k),[],matrix)
    f.close()
except:
    os.remove("./temp.txt")
    quit()

resp=[] ## reading matrix
M=[]
br=0
f1=open("temp.txt", "r")
for line in f1:
    br=br+1
    line=line.replace("[", "")
    line=line.replace("]", "")
    vc=line.split(',')
    resp.append(vc[:])
    M.append(len(vc))
f1.close()

os.remove("./temp.txt")

maxi=M.index(max(M))

clique=resp[maxi]
for i in range(len(clique)):
    clique[i]=int(clique[i])

f = open(cliqueOut ,"w")
for i in clique:
    f.write(str(i)+"\n")
f.close()


f = open(cliqueSequence,"w")
cnt = 0
for i in range(len(seq)):
    if i not in clique:
        cnt+=1
        print(seq[i])
        f.write(seq[i]+"\n")

print("Thrown out: " + str(cnt))
f.write("Thrown out: " + str(cnt)+"\n")
print("Final list of positives:")
f.write("Final list of positives:\n")
for i in range(len(seq)):
    if i in clique:
        print(seq[i])
        f.write(seq[i]+"\n")
f.close()
